import bpy
import os
import sys
import ntpath


def setPath():
    """ Add the path of the current blend file to sys.path """

    path = os.path.dirname(__file__)
    head, tail = ntpath.split(path)
    if head not in sys.path:
        sys.path.append(head)
    return head

redMaterial = bpy.data.materials.new('Material Red')
redMaterial.diffuse_color = (1, 0, 0)

greenMaterial = bpy.data.materials.new('Material Green')
greenMaterial.diffuse_color = (0, 1, 0)

blueMaterial = bpy.data.materials.new('Material Blue')
blueMaterial.diffuse_color = (0, 0, 1)

blackMaterial = bpy.data.materials.new('Material Black')
blackMaterial.diffuse_color = (0, 0, 0)


def readMoleculeFile(fileName):
    moleculeInfo = []
    with open(fileName, 'r') as moleculeFile:
        for line in moleculeFile.readlines():
            listOfWords = line.split()
            if listOfWords[0] == 'ATOM':
                moleculeInfo.append(listOfWords)
    return moleculeInfo


def betterParser(fileName):
    ''' A bit better parser, only works when files really adhere to pdb
        format. Should add some catches, to prevent bugs.
    '''

    moleculeInfo = []
    with open(fileName, 'r') as moleculeFile:
        for line in moleculeFile.readlines():
            oneAtom = {}
            if line[0:4] == 'ATOM':
                oneAtom['x'] = float(line[30:38])
                oneAtom['y'] = float(line[38:46])
                oneAtom['z'] = float(line[46:54])
                oneAtom['element'] = line[76:78].strip()
                moleculeInfo.append(oneAtom)

    return moleculeInfo


def drawMolecule(moleculeInfo):
    for atomInfo in moleculeInfo:
        lx = float(atomInfo['x'])
        ly = float(atomInfo['y'])
        lz = float(atomInfo['z'])

        element = atomInfo['element']
        if element == 'H':
            material = blueMaterial
            atomSize = 0.32
        elif element == 'C':
            material = greenMaterial
            atomSize = 0.91
        elif element == 'N':
            material = blueMaterial
            atomSize = 0.75
        elif element == 'P':
            material = redMaterial
            atomSize = 1.06
        elif element == 'O':
            material = redMaterial
            atomSize = 0.73
        else:
            material = blackMaterial
            atomSize = 1.0
        # Objekt erzeugen
        bpy.ops.mesh.primitive_uv_sphere_add(segments=32,
                                             ring_count=18,
                                             size=atomSize,
                                             location=(lx, ly, lz))
        obj = bpy.context.object
        # Objekt mit Material versehen
        obj.data.materials.append(material)
        # Oberfläche glätten
        bpy.ops.object.shade_smooth()
        obj.name = element + "_" + atomInfo['element']


if __name__ == '__main__':
    folder = setPath()
    # Switch to objekt mode
    if bpy.ops.object.mode_set.poll():
        bpy.ops.object.mode_set(mode='OBJECT')
    # select and delete all objcts from type "Mesh"
    bpy.ops.object.select_by_type(type='MESH')
    bpy.ops.object.delete()

    # list of examples
    # the files are saved in a folder named "molecules"

    # Structure: filename (without file extension), comment
    filelist = [("my_data", "Experiments ..."),
                ("H2O", "Water"),
                ("DNA", "Part of a  DNA"),
                ("methanol", "Methanol"),
                ]

    # my_data = 0, H2O = 1 ...
    selection = 3

    data = betterParser("{}/molecules/{}.pdb".format(folder,
                                                     filelist[selection][0]
                                                     ))
    drawMolecule(data)
